;; MyInt: This sort represents 8-bit bitvecs. We use 8 bits instead of the
;; normal 32 because the printed values are easier to read! All the same
;; general principals apply.
(define-sort MyInt () (_ BitVec 8))

;; These represent the parameters to our method.
(declare-const a MyInt)
(declare-const b MyInt)
(declare-const c MyInt)

;; Here we do some SSA style assignments to get the final value of trian. If you
;; don't know about SSA (Singal Static Assignment), don't worry! It's a fancy
;; way of saying that we turn code with changing variable values like:
;;
;;     int a = 1;
;;     a = a + 1;
;;     a = a + 1;
;;
;; into code without any side effects like:
;;
;;     int a1 = 1;
;;     int a2 = a1 + 1;
;;     int a3 = a2 + 1;
;;
;; Since the `trian` variable gets updated throughout the `classify` method,
;; we break it up into `trian`, `trian2`, and `trian3`, corresponding to the
;; amount added to `trian` on lines 25, 28, and 31 respectively.
(declare-const trian1 MyInt)
(declare-const trian2 MyInt)
(declare-const trian3 MyInt)

;; The final value of `trian` after line 32.
(declare-const trian  MyInt)

; Conditionals in if statements given by their line numbers in the initial
; source code. Thus line-20-cond corresponds to
;
;       if (a <= 0 || b <= 0 || c <= 0) { ...
;
; This is provided for convenience

(declare-const line-20-cond bool) ; if (a <= 0 || b <= 0 || c <= 0)
(declare-const line-24-cond bool) ; if (a == b)
(declare-const line-27-cond bool) ; if (a == c)
(declare-const line-30-cond bool) ; if (b == c)
(declare-const line-33-cond bool) ; if (trian == 0)
(declare-const line-34-cond bool) ; if (a + b <= c || a + c <= b || b + c <= a)

; Assign the values of the conditionals
(assert (=  line-20-cond (or  (bvsle a #x00)
                              (bvsle b #x00)
                              (bvsle c #x00))))

(assert (= trian1 (if_then_else line-24-cond #x01 #x00)))
(assert (= trian2 (if_then_else line-27-cond #x02 #x00)))
(assert (= trian3 (if_then_else line-30-cond #x03 #x00)))

(assert (= trian (bvadd trian1 trian2 trian3)))

(declare-const initial-condition bool)
(declare-const mutated-condition bool)

;; MUTANT 84
;; @@ -34 +34 @@
;; -            if (a + b <= c || a + c <= b || b + c <= a) {
;; +            if (a + b <= c || a + c <= b || b + c < a) {

;;;;;;;;;;;;;;;;; START STUDENT CODE ;;;;;;;;;;;;;;;

;; TODO 1: Much of what you did for `pair2` applies here. Copy/paste applicable
;;     constraints that you wrote for `pair2` below


;; TODO 2: The mutant is on line 34. If the mutant is to behave differently than
;;     the original program, this mutation needs to be executed. With this in
;;     mind, what can we conclude about the condition on line 33? Issue a
;;     constraint to enforce this.


;; TODO 3: The original condition on line 34 is of the form
;;
;;         `A || B || C1`,
;;
;;     and the mutant is of the form
;;
;;         `A || B || C2`,
;;
;;     where
;;
;;         A  := `a + b <= c`,
;;         B  := `a + c <= b`,
;;         C1 := `b + c <= a`,
;;         C2 := `b + c <  a`.
;;
;;     Recalling that the `||` operator is short circuiting, what must be true
;;     about the truthiness of expressions `A` and `B` whenever the mutated
;;     expression `C2` is executed? Use this observation to issue constraints
;;     for expressions `A` and `B`


;; TODO 3: Suppose expressions `C1` and `C2` both execute, and suppose that
;;     these executions lead to the original and the mutated method returning
;;     different values. What can you conclude about the values of `C1` and
;;     `C2`? Issue a constraint capturing this behavior.


;; TODO 4: Run this file through Z3...is the mutant equivalent, or no?

;;;;;;;;;;;;;;;;; END STUDENT CODE ;;;;;;;;;;;;;;;

(check-sat)
;; If needed, run
;; (echo "-------------------------------- Getting model ---------------------------------")
;; (get-model)

